package jumbo.xml;

/** an attribute as defined in the spec (not an AttDef). Consists
of a name, value and type (all can be returned by SAX). The type is
held as an integer (see below). In general names and values are exact
but some XML operations allow for variation.
<BR>
Most operations involving attributes will be found under XNode
@author P.Murray-Rust, 1998
*/
public class Attribute extends XNode {

/** CDATA or unknown */
	public final static int CDATA    = 0;
	public final static int ID       = 1;
	public final static int IDREF    = 2;
	public final static int IDREFS   = 3;
	public final static int NMTOKEN  = 4;
	public final static int NMTOKENS = 5;
	public final static int NOTATION = 6;
	public final static int NDATA	 = 7;

	public final static String typeString[] = {
		"CDATA",
	    "ID",
	    "IDREF",
	    "IDREFS",
	    "NMTOKEN",
	    "NMTOKENS",
	    "NOTATION",
	    "NDATA",
	};

	String name;
	String value = null;
	int type = CDATA;
//	String defaultValue = null;
//	boolean implied = false;

/** make attribute 
@param name name (not checked for XML validity)
@param value value (may contain '"&<>)
@param type type of attribute (see symbolic values)
*/
	public Attribute(String name, String value, int type) {
		this.name = name;
		this.value = value;
		this.type = type;
	}
	
/** create Attribute from name and value. Assume type is CDATA
@param name name (not checked for XML validity)
@param value value (may contain '"&<>)
*/
	public Attribute(String name, String value) {
		this(name, value, CDATA);
	}
	
/** copy constructor
@param Attribute attribute the attribute to be copied
*/
	public Attribute(Attribute attribute) {
		this(attribute.name, attribute.value, attribute.type);
	}
	
/** return attribute as "name=value" (no spaces). The famous five
characters are escaped (' -> &apos; etc.)
@return Stringattribute as name=value
*/
	public String toString() {
		return name+"=\""+Util.escape(value)+"\"";
	}
}