package jumbo.xml;

import java.applet.Applet;

import java.io.BufferedInputStream;
import java.io.EOFException;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;

import java.net.MalformedURLException;
import java.net.URL;

import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

//import jumbo.simplegraph.SimpleGraph;

/**
A number of miscellaneous tools. Originally devised for jumbo.sgml,
now rewritten for jumbo.xml. Use these at your peril - some will
be phased out
@author (C) P. Murray-Rust, 1998
*/
public class Util {

    public static final String SLASH = "/";
// default o/p stream
    static boolean stdout = true;
    static boolean stderr = true;

    static String blank20 = "                    ";

/** offers a prompt on stdout and reads the input until CR */
/*
    public static String prompt(String prompt) {
        System.out.print(prompt+": ");
        System.out.flush();
        StringBuffer s = new StringBuffer();
        try {
            while (true) {
                int c = System.in.read();
                if (c == '\n') break;
                char cc = (char) c;
                s.append(cc);
            }
        } catch (Exception e) {
            System.err.println("Util.prompt error: " + e);
        }
        return s.toString();
    }
*/

/** make a String of a given number of spaces 
*/
    public static String spaces(int nspace) {
        if (nspace <= 0) {
            return "";
        } else if (nspace <= 20) {
            return blank20.substring(0, nspace);
        } else {
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < nspace; i++) {
                sb.append(" ");
            }
            return sb.toString();
        }
    }

/** remove balanced quotes from ends of (trimmed) string, else no action
*/
    public static String deQuote(String s) {
        String ss = s.trim();
        char c = ss.charAt(0);
        if (c == '"' || c == '\'') {
            int l = ss.length();
            if (ss.charAt(l-1) == c) {
                return ss.substring(1, l-1);
            }
        }
        return s;
    }

/** return index of balanced bracket; -1 for none. String MUST start
with '('
*/
    public static int indexOfBalancedBracket(char lbrack, String s) {
        if (s.charAt(0) != lbrack) return -1;
        char rbrack = ' ';
        if (lbrack == '(') {
            rbrack = ')';
        } else if (lbrack == '<') {
            rbrack = '>';
        } else if (lbrack == '[') {
            rbrack = ']';
        } else if (lbrack == '{') {
            rbrack = '}';
        }
        int l = s.length();
        int i = 0;
        int level = 0;
        while (i < l) {
            if (s.charAt(i) == lbrack) {
                level++;
            } else if(s.charAt(i) == rbrack) {
                level--;
                if (level == 0) return i;
            }
            i++;
        }
        return -1;
    }
            
/** concatenate strings into quote-separated string
@param String[] s strings to be concatenated
@return String concatenated string 
*/
    public static String quoteConcatenate(String[] s) {
        StringBuffer sb = new StringBuffer();
            
        for (int i = 0; i < s.length; i++) {
            if (i > 0) sb.append(" ");
            boolean quote = false;
            if (s[i].indexOf(" ") != -1) {
                sb.append("\"");
                quote = true;
            }
            sb.append(s[i]);
            if (quote) sb.append("\"");
        }
               
        return sb.toString();
    }

/** splits a whitespace-separated set of tokens into a String[]
@param String s string to be split
@return String[] result of splitting (null if s==null)
*/
	public static String[] split(String s) {
		if (s == null) return null;
		StringTokenizer st = new StringTokenizer(s, " \n\t\r");
		String[] out = new String[st.countTokens()];
		int count = 0;
		while (st.hasMoreTokens()) {
			out[count++] = (String) st.nextToken();
		}
		return out;
	}

/** remove balanced (well-formed) markup from a string. Crude (i.e.
not fully XML-compliant);</BR>
Example: "This is &lt;A HREF="foo"&gt;bar&lt;/A&gt; and &lt;/BR&gt; a break" goes to
"This is bar and a break"
*/
	public static String removeHTML(String s) {
		int idx = s.indexOf("<");
		if (idx == -1) return s;
		if (s.startsWith("<")) {
			idx = s.indexOf(">");
			s = s.substring(idx+1);
		}
		StringTokenizer st = new StringTokenizer(s);
		String ss = "";
		while (st.hasMoreTokens()) {
			ss += st.nextToken("<");
			if (!st.hasMoreTokens()) break;
			st.nextToken(">");
		}
		return ss;
	}

/** record that we have hit a program bug!!! 
*/
    public static void bug(String s) {
            bug(new Exception(s));
    }

    public static void bug(Exception e) {
        System.err.println
            ("SERIOUS PROGRAM ERROR! - please report the following:");
        e.printStackTrace(System.err);
    }



/** get a long from a Date (should be easy, but isn't)
*/
    public static long longFromDate(Date d) {
        if (d == null) return -1;
        return Date.UTC(
            d.getYear(),
            d.getMonth(),
            d.getDate(),
            d.getHours(),
            d.getMinutes(),
            d.getSeconds()
        );
    }

/** create from a date; if hrs, sec are negative, they are ignored
*/
    public static Date getDate
        (int year, int month, int day, int hrs, int min, int sec) {
        Date date = null;
        if (hrs < 0) {
            date = new Date(year, month, day);
        } else if (sec < 0) {
            date = new Date(year, month, day, hrs, min);
        } else {
            date = new Date(year, month, day, hrs, min, sec);
        }
        return date;
    }

/** get current directory
*/
    public static String getPWDName() {
        File f = new File(".");
        return new File(f.getAbsolutePath()).getParent();
    }

/** make substitutions in a string. If oldSubtrings = {"A", "BB", "C"}
and newSubstrings = {"aa", "b", "zz"} then every occurrence of 
"A" in s is replaced with "aa", etc. "BBB" would be replaced by "bB"
*/
    public static String substituteStrings
        (String s, String[] oldSubstrings, String[] newSubstrings) {
        if (oldSubstrings.length != newSubstrings.length) {
            throw new IllegalArgumentException(
        "Util.substituteStrings  arguments of different lengths");
        }
        for (int i = 0; i < oldSubstrings.length; i++) {
            String oldS = oldSubstrings[i];
            String newS = newSubstrings[i];
            int lo = oldS.length();
            if (s.indexOf(oldS) == -1) continue;
			String ss = "";
			while (true) {
				int idx = s.indexOf(oldS);
                if (idx == -1) {
					ss += s;
					break;
				}
                ss += s.substring(0, idx)+newS;
				s = s.substring(idx+lo);
            }
			s = ss;
        }
        return s;
    }

/** capitalise a String (whatever the starting case)
*/
    public static String capitalise(String s) {
        if (s.equals("")) return "";
        if (s.length() == 1) {
            return s.toUpperCase();
        } else {
            return s.substring(0,1).toUpperCase() + 
                s.substring(1).toLowerCase();
        }
    }

	public static String toCamelCase(String s) {
		StringTokenizer st = new StringTokenizer(s, " \n\r\t");
		String out = "";
		while (st.hasMoreTokens()) {
			s = st.nextToken();
			if (out != "") {
				s = capitalise(s);
			}
			out += s;
		}
		return out;
	}

// note & must come first to avoid recursion
//	static String[] escapes = {"&",     "'",      "\"",     "<",    ">"};
//	static String[] escapel = {"&amp;", "&apos;", "&quot;", "&lt;", "&gt;"};
// this gave trouble with apos, so try the minimum

	static char[] escapes = {'&',  /*   '\'',  */    '"',     '<',  /*  '>'*/};
	static String[] escapel = {"&amp;", /* "&apos;", */ "&quot;", "&lt;",/* "&gt;"*/};
/** escape characters in an XML string (' -> &apos; , etc); also escape non-XML
characters
*/
	public static String escape(String s) {
		int l = s.length();
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < l; i++) {
			char ch = s.charAt(i);
			for (int j = 0; j < escapes.length; j++) {
				if (ch == escapes[j]) {
					sb.append(escapel[j]);
					continue;
				}
			}
			int ich = (int) ch;
			if (ich >= 32 && ich <= 127) {
				sb.append(ch);
// tab
			} else if (ch == '\t') {
				sb.append(ch);
// nl
			} else if (ch == '\n') {
				sb.append(ch);
// cr
			} else if (ch == '\r') {
				sb.append(ch);
			} else {
				System.out.println("Non-XML char: "+ich);
				sb.append("&#"+ich+";");
			}
		}
		s = sb.toString();
		return s;
	}

/** runs the garbage collector if memory drops below mem.  (I use a value
of 300000 - your mileage may vary).  Potentially used in loops for processing
input and creation of objects
*/
    public static void freeMemory(long mem) {
        if (Runtime.getRuntime().freeMemory() < mem) {
            System.gc();
            System.out.println("Now Free "+Runtime.getRuntime().freeMemory());
        }
    }

/** horrid, but I couldn't find if Java reads hex.    Crude; no exceptions thrown
*/
    public static int getIntFromHex(String hex) {
        if (hex.startsWith("0X") || hex.startsWith("0x")) {
            hex = hex.substring(2);
        } else if (hex.startsWith("X") || hex.startsWith("x")) {
            hex = hex.substring(1);
        }
        int result = 0;
        for (int i = 0; i < hex.length(); i++) {
            char c = hex.charAt(i);
            int j = (int) c;
            if (c >= 97) {
                c -= 87;
            } else if (c >= 65) {
                c -= 55;
            } else {
                c -= 48;
            }
            result = 16*result + c;
        }
        return result;
    }
/** reads a byte array from file, *including* line feeds
*/
    public static byte[] readByteArray(String filename)
        throws FileNotFoundException, IOException {
        DataInputStream dis = new DataInputStream(
            new FileInputStream(filename));
        return Util.readByteArray(dis);
    }


/** reads a byte array from DataInputStream, *including* line feeds
*/
    public static byte[] readByteArray(DataInputStream d) 
        throws IOException {
        int len = 100;
        int count = 0;
        byte[] src = new byte[len];
        byte b;
        while (true) {
            try {
                b = d.readByte();
            } catch (EOFException e) {
                break;
            }
            src[count] = b;
            if (++count >= len) {
                len *= 2;
                byte[] temp = new byte[len];
                System.arraycopy(src, 0, temp, 0, count);
                src = temp;
            }
        }
        len = count;
        byte[] temp = new byte[len];
        System.arraycopy(src, 0, temp, 0, count);
        return temp;
    }

/** strip linefeeds from a byte array
*/
    public static byte[] stripNewlines(byte[] b) {
        int l = b.length;
        byte[] bb = new byte[l];
        int j = 0;
        for (int i = 0; i < l; i++) {
            if (b[i] != '\n') {
                bb[j++] = b[i];
            }
        }
        byte[] bbb = new byte[j];
        System.arraycopy(bb, 0, bbb, 0, j);
        return bbb;
    }

/** converts a string to its byte[] equivalent
*/
    public static byte[] string2Bytes(String s) {
        int l = s.length();
        byte[] b = new byte[l];
        s.getBytes(0, l, b, 0);
        return b;
    }

/** a crude way of identifying a right mouse click (because I
left the Java book behind)
*/
	public static boolean isRightMouseClick(java.awt.event.MouseEvent event) {
		String s = event.toString();
		return (s.indexOf("MOUSE_CLICKED") != -1 && 
			s.indexOf("mods=4") != -1);
	}

	static Hashtable appletTable = new Hashtable();
/** record that an object is an applet rather than an application.
call from applet's init() method
*/
	public static void registerApplet(Applet applet) {
		appletTable.put(applet, applet);
	}


/** retrive that an object is an applet rather than an application.
must have been registered above 
*/
	public static boolean isRegisteredApplet(Applet applet) {
		return (appletTable.get(applet) != null);
	}

/** If a URL is relative, make it absolute against the current directory.
If url already has a protocol, return unchanged 
*/
    public static String makeAbsoluteURL (String url)
        throws java.net.MalformedURLException {
        URL baseURL;
        String fileSep = System.getProperty("file.separator");
// why the final slash?
//		url = url.replace(fileSep.charAt(0), '/') + '/';
        url = url.replace(fileSep.charAt(0), '/');

// is url alreday a valid URL?
		boolean ok = true;
		try {
			URL u = new URL(url);
		} catch (MalformedURLException mue) {
			ok = false;
// DOS filenames (e.g. C:\foo) gives problems
			String mueString = mue.toString().trim();
			int idx = mueString.indexOf("unknown protocol:");
			if (idx != -1) {
				mueString = mueString.substring(idx+"unknown protocol:".length()).trim();
				if (mueString.length() == 1) {
					throw new MalformedURLException
						("***Must precede DOS filenames with 'file:/'");
				}
			}
		}
		if (ok) {
			return url;
		}

		String currentDirectory = System.getProperty("user.dir");
        String file = currentDirectory.replace(fileSep.charAt(0), '/') + '/';

        if (file.charAt(0) != '/') {
            file = "/" + file;
        }
        baseURL = new URL("file", null, file);

        String newUrl = new URL(baseURL, url).toString();
        return newUrl;
    }

/** If a URL is relative, make it absolute against either the current 
directory (application) or codebase (applet)
*/
    public static String makeAbsoluteURL (Applet applet, String url)
        throws java.net.MalformedURLException {
        URL baseURL;
        if (!Util.isRegisteredApplet(applet)) {
			return Util.makeAbsoluteURL(url);
		}
		baseURL  = applet.getCodeBase();
        return new URL(baseURL, url).toString();
    }
}